"""Remote client command for export container filesystem to tarball."""
import sys

import podman
from pypodman.lib import AbstractActionBase


class Export(AbstractActionBase):
    """Class for exporting container filesystem to tarball."""

    @classmethod
    def subparser(cls, parent):
        """Add Export command to parent parser."""
        parser = parent.add_parser(
            'export',
            help='export container to tarball',
        )
        parser.add_argument(
            '--output',
            '-o',
            metavar='PATH',
            nargs=1,
            required=True,
            help='Write to this file on host',
        )
        parser.add_argument(
            'container',
            nargs=1,
            help='container to use as source',
        )
        parser.set_defaults(class_=cls, method='export')

    def export(self):
        """Create tarball from container filesystem."""
        try:
            try:
                ctnr = self.client.containers.get(self._args.container[0])
            except podman.ContainerNotFound as e:
                sys.stdout.flush()
                print(
                    'Container {} not found.'.format(e.name),
                    file=sys.stderr,
                    flush=True)
                return 1
            else:
                ctnr.export(self._args.output[0])
        except podman.ErrorOccurred as e:
            sys.stdout.flush()
            print(
                '{}'.format(e.reason).capitalize(),
                file=sys.stderr,
                flush=True)
            return 1
        return 0



"""Remote client command for creating image from container."""
import sys

import podman
from pypodman.lib import AbstractActionBase, ChangeAction


class Commit(AbstractActionBase):
    """Class for creating image from container."""

    @classmethod
    def subparser(cls, parent):
        """Add Commit command to parent parser."""
        parser = parent.add_parser(
            'commit',
            help='create image from container',
        )
        parser.add_argument(
            '--author',
            help='Set the author for the committed image',
        )
        parser.add_argument(
            '--change',
            '-c',
            action=ChangeAction,
        )
        parser.add_argument(
            '--format',
            '-f',
            choices=('oci', 'docker'),
            default='oci',
            type=str.lower,
            help='Set the format of the image manifest and metadata.'
            ' (Ignored.)',
        )
        parser.add_argument(
            '--iidfile',
            metavar='PATH',
            help='Write the image ID to the file',
        )
        parser.add_argument(
            '--message',
            '-m',
            help='Set commit message for committed image'
            ' (Only on docker images.)',
        )
        parser.add_flag(
            '--pause',
            '-p',
            help='Pause the container when creating an image',
        )
        parser.add_flag(
            '--quiet',
            '-q',
            help='Suppress output',
        )
        parser.add_argument(
            'container',
            nargs=1,
            help='container to use as source',
        )
        parser.add_argument(
            'image',
            nargs=1,
            help='image name to create',
        )
        parser.set_defaults(class_=cls, method='commit')

    def commit(self):
        """Create image from container."""
        try:
            try:
                ctnr = self.client.containers.get(self._args.container[0])
            except podman.ContainerNotFound as e:
                sys.stdout.flush()
                print(
                    'Container {} not found.'.format(e.name),
                    file=sys.stderr,
                    flush=True)
                return 1
            else:
                ident = ctnr.commit(
                    self.opts['image'][0],
                    change=self.opts.get('change', None),
                    message=self.opts.get('message', None),
                    pause=self.opts['pause'],
                    author=self.opts.get('author', None),
                )

                if not self.opts['quiet']:
                    print(ident)
        except podman.ErrorOccurred as e:
            sys.stdout.flush()
            print(
                '{}'.format(e.reason).capitalize(),
                file=sys.stderr,
                flush=True)
            return 1
        return 0



"""
Supplimental argparse.Action converters and validaters.

The constructors are very verbose but remain for IDE support.
"""
import argparse
import copy
import os
import signal

# API defined by argparse.Action therefore shut up pylint
# pragma pylint: disable=redefined-builtin
# pragma pylint: disable=too-few-public-methods
# pragma pylint: disable=too-many-arguments


class ChangeAction(argparse.Action):
    """Convert and validate change argument."""

    def __init__(self,
                 option_strings,
                 dest,
                 nargs=None,
                 const=None,
                 default=None,
                 type=None,
                 choices=None,
                 required=False,
                 help=None,
                 metavar='OPT=VALUE'):
        """Create ChangeAction object."""
        help = (help or '') + ('Apply change(s) to the new image.'
                               ' May be given multiple times.')
        if default is None:
            default = []

        super().__init__(
            option_strings=option_strings,
            dest=dest,
            nargs=nargs,
            const=const,
            default=default,
            type=type,
            choices=choices,
            required=required,
            help=help,
            metavar=metavar)

    def __call__(self, parser, namespace, values, option_string=None):
        """Convert and Validate input."""
        items = getattr(namespace, self.dest, None) or []
        items = copy.copy(items)

        choices = ('CMD', 'ENTRYPOINT', 'ENV', 'EXPOSE', 'LABEL', 'ONBUILD',
                   'STOPSIGNAL', 'USER', 'VOLUME', 'WORKDIR')

        opt, _ = values.split('=', 1)
        if opt not in choices:
            parser.error('Option "{}" is not supported by argument "{}",'
                         ' valid options are: {}'.format(
                             opt, option_string, ', '.join(choices)))
        items.append(values)
        setattr(namespace, self.dest, items)


class SignalAction(argparse.Action):
    """Validate input as a signal."""

    def __init__(self,
                 option_strings,
                 dest,
                 nargs=None,
                 const=None,
                 default=None,
                 type=str,
                 choices=None,
                 required=False,
                 help='The signal to send.'
                 ' It may be given as a name or a number.',
                 metavar='SIGNAL'):
        """Create SignalAction object."""
        super().__init__(
            option_strings=option_strings,
            dest=dest,
            nargs=nargs,
            const=const,
            default=default,
            type=type,
            choices=choices,
            required=required,
            help=help,
            metavar=metavar)

        if hasattr(signal, "Signals"):

            def _signal_number(signame):
                cooked = 'SIG{}'.format(signame)
                try:
                    return signal.Signals[cooked].value
                except ValueError:
                    pass
        else:

            def _signal_number(signame):
                cooked = 'SIG{}'.format(signame)
                for n, v in sorted(signal.__dict__.items()):
                    if n != cooked:
                        continue
                    if n.startswith("SIG") and not n.startswith("SIG_"):
                        return v

        self._signal_number = _signal_number

    def __call__(self, parser, namespace, values, option_string=None):
        """Validate input is a signal for platform."""
        if values.isdigit():
            signum = int(values)
            if signal.SIGRTMIN <= signum >= signal.SIGRTMAX:
                raise ValueError('"{}" is not a valid signal. {}-{}'.format(
                    values, signal.SIGRTMIN, signal.SIGRTMAX))
        else:
            signum = self._signal_number(values)
            if signum is None:
                parser.error(
                    '"{}" is not a valid signal,'
                    ' see your platform documentation.'.format(values))
        setattr(namespace, self.dest, signum)


class UnitAction(argparse.Action):
    """Validate number given is positive integer, with optional suffix."""

    def __init__(self,
                 option_strings,
                 dest,
                 nargs=None,
                 const=None,
                 default=None,
                 type=None,
                 choices=None,
                 required=False,
                 help=None,
                 metavar='UNIT'):
        """Create UnitAction object."""
        help = (help or metavar or dest)\
            + ' (format: <number>[<unit>], where unit = b, k, m or g)'
        super().__init__(
            option_strings=option_strings,
            dest=dest,
            nargs=nargs,
            const=const,
            default=default,
            type=type,
            choices=choices,
            required=required,
            help=help,
            metavar=metavar)

    def __call__(self, parser, namespace, values, option_string=None):
        """Validate input as a UNIT."""
        try:
            val = int(values)
        except ValueError:
            if not values[:-1].isdigit():
                msg = ('{} must be a positive integer,'
                       ' with optional suffix').format(option_string)
                parser.error(msg)
            if not values[-1] in ('b', 'k', 'm', 'g'):
                msg = '{} only supports suffices of: b, k, m, g'.format(
                    option_string)
                parser.error(msg)
        else:
            if val <= 0:
                msg = '{} must be a positive integer'.format(option_string)
                parser.error(msg)

        setattr(namespace, self.dest, values)


class PositiveIntAction(argparse.Action):
    """Validate number given is positive integer."""

    def __init__(self,
                 option_strings,
                 dest,
                 nargs=None,
                 const=None,
                 default=None,
                 type=int,
                 choices=None,
                 required=False,
                 help='Must be a positive integer.',
                 metavar=None):
        """Create PositiveIntAction object."""
        super().__init__(
            option_strings=option_strings,
            dest=dest,
            nargs=nargs,
            const=const,
            default=default,
            type=type,
            choices=choices,
            required=required,
            help=help,
            metavar=metavar)

    def __call__(self, parser, namespace, values, option_string=None):
        """Validate input."""
        if values > 0:
            setattr(namespace, self.dest, values)
            return

        msg = '{} must be a positive integer'.format(option_string)
        parser.error(msg)


class PathAction(argparse.Action):
    """Expand user- and relative-paths."""

    def __init__(self,
                 option_strings,
                 dest,
                 nargs=None,
                 const=None,
                 default=None,
                 type=None,
                 choices=None,
                 required=False,
                 help=None,
                 metavar='PATH'):
        """Create PathAction object."""
        super().__init__(
            option_strings=option_strings,
            dest=dest,
            nargs=nargs,
            const=const,
            default=default,
            type=type,
            choices=choices,
            required=required,
            help=help,
            metavar=metavar)

    def __call__(self, parser, namespace, values, option_string=None):
        """Resolve full path value on local filesystem."""
        setattr(namespace, self.dest,
                os.path.abspath(os.path.expanduser(values)))



"""Remote client command for pod subcommands."""
import inspect
import logging
import sys

from pypodman.lib import AbstractActionBase

# pylint: disable=wildcard-import
# pylint: disable=unused-wildcard-import
from .pod import *


class Pod(AbstractActionBase):
    """Class for creating a pod."""

    @classmethod
    def subparser(cls, parent):
        """Add Pod Create command to parent parser."""
        pod_parser = parent.add_parser(
            'pod',
            help='pod commands.'
            ' For subcommands, see: %(prog)s pod --help')
        subparser = pod_parser.add_subparsers()

        # pull in plugin(s) code for each subcommand
        for name, obj in inspect.getmembers(
                sys.modules['pypodman.lib.actions.pod'],
                predicate=inspect.isclass):
            if hasattr(obj, 'subparser'):
                try:
                    obj.subparser(subparser)
                except NameError as e:
                    logging.critical(e)
                    logging.warning(
                        'See subparser configuration for Class "%s"', name)
                    sys.exit(3)



"""Remote client command for searching registries for an image."""
import argparse
import sys
from collections import OrderedDict

import podman
from pypodman.lib import (AbstractActionBase, PositiveIntAction, Report,
                          ReportColumn)


class FilterAction(argparse.Action):
    """Parse filter argument components."""

    def __init__(self,
                 option_strings,
                 dest,
                 nargs=None,
                 const=None,
                 default=None,
                 type=None,
                 choices=None,
                 required=False,
                 help=None,
                 metavar='FILTER'):
        """Create FilterAction object."""
        help = (help or '') + (' (format: stars=##'
                               ' or is-automated=[True|False]'
                               ' or is-official=[True|False])')
        super().__init__(
            option_strings=option_strings,
            dest=dest,
            nargs=nargs,
            const=const,
            default=default,
            type=type,
            choices=choices,
            required=required,
            help=help,
            metavar=metavar)

    def __call__(self, parser, namespace, values, option_string=None):
        """
        Convert and Validate input.

        Note: side effects
        1) self.dest value is set to subargument dest
        2) new attribute self.dest + '_value' is created with 2nd value.
        """
        opt, val = values.split('=', 1)
        if opt == 'stars':
            msg = ('{} option "stars" requires'
                   ' a positive integer').format(self.dest)
            try:
                val = int(val)
            except ValueError:
                parser.error(msg)

            if val < 0:
                parser.error(msg)
        elif opt == 'is-automated':
            if val.capitalize() in ('True', 'False'):
                val = bool(val)
            else:
                msg = ('{} option "is-automated"'
                       ' must be True or False.'.format(self.dest))
                parser.error(msg)
        elif opt == 'is-official':
            if val.capitalize() in ('True', 'False'):
                val = bool(val)
            else:
                msg = ('{} option "is-official"'
                       ' must be True or False.'.format(self.dest))
                parser.error(msg)
        else:
            msg = ('{} only supports one of the following options:\n'
                   '  stars, is-automated, or is-official').format(self.dest)
            parser.error(msg)
        setattr(namespace, self.dest, opt)
        setattr(namespace, self.dest + '_value', val)


class Search(AbstractActionBase):
    """Class for searching registries for an image."""

    @classmethod
    def subparser(cls, parent):
        """Add Search command to parent parser."""
        parser = parent.add_parser('search', help='search for images')
        super().subparser(parser)
        parser.add_argument(
            '--filter',
            '-f',
            action=FilterAction,
            help='Filter output based on conditions provided.')
        parser.add_argument(
            '--limit',
            action=PositiveIntAction,
            default=25,
            help='Limit the number of results.'
            ' (default: %(default)s)')
        parser.add_argument('term', nargs=1, help='search term for image')
        parser.set_defaults(class_=cls, method='search')

    def __init__(self, args):
        """Construct Search class."""
        super().__init__(args)

        self.columns = OrderedDict({
            'name':
            ReportColumn('name', 'NAME', 44),
            'description':
            ReportColumn('description', 'DESCRIPTION', 44),
            'star_count':
            ReportColumn('star_count', 'STARS', 5),
            'is_official':
            ReportColumn('is_official', 'OFFICIAL', 8),
            'is_automated':
            ReportColumn('is_automated', 'AUTOMATED', 9),
        })

    def search(self):
        """Search registries for image."""
        try:
            rows = list()
            for entry in self.client.images.search(
                    self._args.term[0], limit=self._args.limit):

                if self._args.filter == 'is-official':
                    if self._args.filter_value != entry.is_official:
                        continue
                elif self._args.filter == 'is-automated':
                    if self._args.filter_value != entry.is_automated:
                        continue
                elif self._args.filter == 'stars':
                    if self._args.filter_value > entry.star_count:
                        continue

                fields = dict(entry._asdict())

                status = '[OK]' if entry.is_official else ''
                fields['is_official'] = status

                status = '[OK]' if entry.is_automated else ''
                fields['is_automated'] = status

                if self._args.truncate:
                    fields.update({'name': entry.name[-44:]})
                rows.append(fields)

            with Report(self.columns, heading=self._args.heading) as report:
                report.layout(
                    rows, self.columns.keys(), truncate=self._args.truncate)
                for row in rows:
                    report.row(**row)
        except podman.ErrorOccurred as e:
            sys.stdout.flush()
            print(
                '{}'.format(e.reason).capitalize(),
                file=sys.stderr,
                flush=True)



"""Remote client command for starting containers."""
import sys

import podman
from pypodman.lib import AbstractActionBase


class Start(AbstractActionBase):
    """Class for starting container."""

    @classmethod
    def subparser(cls, parent):
        """Add Start command to parent parser."""
        parser = parent.add_parser('start', help='start container')
        parser.add_flag(
            '--attach',
            '-a',
            help="Attach container's STDOUT and STDERR.")
        parser.add_argument(
            '--detach-keys',
            metavar='KEY(s)',
            default=4,
            help='Override the key sequence for detaching a container.'
            ' (format: a single character [a-Z] or ctrl-<value> where'
            ' <value> is one of: a-z, @, ^, [, , or _) (default: ^D)')
        parser.add_flag(
            '--interactive',
            '-i',
            help="Attach container's STDIN.")
        # TODO: Implement sig-proxy
        parser.add_flag(
            '--sig-proxy',
            help="Proxy received signals to the process."
        )
        parser.add_argument(
            'containers',
            nargs='+',
            help='containers to start',
        )
        parser.set_defaults(class_=cls, method='start')

    def start(self):
        """Start provided containers."""
        stdin = sys.stdin if self.opts['interactive'] else None
        stdout = sys.stdout if self.opts['attach'] else None

        try:
            for ident in self._args.containers:
                try:
                    ctnr = self.client.containers.get(ident)
                    ctnr.attach(
                        eot=self.opts['detach_keys'],
                        stdin=stdin,
                        stdout=stdout)
                    ctnr.start()
                except podman.ContainerNotFound as e:
                    sys.stdout.flush()
                    print(
                        'Container "{}" not found'.format(e.name),
                        file=sys.stderr,
                        flush=True)
                else:
                    print(ident)
        except podman.ErrorOccurred as e:
            sys.stdout.flush()
            print(
                '{}'.format(e.reason).capitalize(),
                file=sys.stderr,
                flush=True)
            return 1
        return 0



"""Remote client command for inspecting pods."""
import json
import sys

import podman
from pypodman.lib import AbstractActionBase


class InspectPod(AbstractActionBase):
    """Class for reporting on pods and their containers."""

    @classmethod
    def subparser(cls, parent):
        """Add Pod Inspect command to parent parser."""
        parser = parent.add_parser(
            'inspect',
            help='configuration and state information about a given pod')
        parser.add_argument('pod', nargs='+', help='pod(s) to inspect')
        parser.set_defaults(class_=cls, method='inspect')

    def inspect(self):
        """Report on provided pods."""
        output = {}
        try:
            for ident in self._args.pod:
                try:
                    pod = self.client.pods.get(ident)
                except podman.PodNotFound:
                    print(
                        'Pod "{}" not found.'.format(ident),
                        file=sys.stdout,
                        flush=True)
                output.update(pod.inspect()._asdict())
        except podman.ErrorOccurred as e:
            sys.stdout.flush()
            print(
                '{}'.format(e.reason).capitalize(),
                file=sys.stderr,
                flush=True)
            return 1
        else:
            print(json.dumps(output, indent=2))
        return 0



"""Remote client command for reporting on pod and container(s)."""
import sys

import podman
from pypodman.lib import AbstractActionBase


class TopPod(AbstractActionBase):
    """Report on containers in Pod."""

    @classmethod
    def subparser(cls, parent):
        """Add Pod Top command to parent parser."""
        parser = parent.add_parser('top', help='report on containers in pod')
        parser.add_argument('pod', nargs=1, help='Pod to report on.')
        parser.set_defaults(class_=cls, method='top')

    def top(self):
        """Report on pod and container(s)."""
        try:
            for ident in self._args.pod:
                pod = self.client.pods.get(ident)
                print(pod.top())
        except podman.PodNotFound as ex:
            print(
                'Pod "{}" not found.'.format(ex.name),
                file=sys.stderr,
                flush=True)
        except podman.ErrorOccurred as ex:
            print(
                '{}'.format(ex.reason).capitalize(),
                file=sys.stderr,
                flush=True)
            return 1
        return 0



import os
from test.podman_testcase import PodmanTestCase

import podman
from podman import FoldedString

pod = None


class TestPodsCtnrs(PodmanTestCase):
    @classmethod
    def setUpClass(cls):
        # Populate storage
        super().setUpClass()

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()

    def setUp(self):
        self.tmpdir = os.environ['TMPDIR']
        self.host = os.environ['PODMAN_HOST']

        self.pclient = podman.Client(self.host)

    def test_010_populate(self):
        global pod

        pod = self.pclient.pods.create('pod1')
        self.assertEqual('pod1', pod.name)

        img = self.pclient.images.get('docker.io/library/alpine:latest')
        ctnr = img.container(pod=pod.id)

        pod.refresh()
        self.assertEqual('1', pod.numberofcontainers)
        self.assertEqual(ctnr.id, pod.containersinfo[0]['id'])

    def test_015_one_shot(self):
        global pod

        details = pod.inspect()
        state = FoldedString(details.containers[0]['state'])
        self.assertEqual(state, 'configured')

        pod = pod.start()
        status = FoldedString(pod.containersinfo[0]['status'])
        # Race on whether container is still running or finished
        self.assertIn(status, ('stopped', 'exited', 'running'))

        pod = pod.restart()
        status = FoldedString(pod.containersinfo[0]['status'])
        self.assertIn(status, ('stopped', 'exited', 'running'))

        # Pod kill is broken, so use stop for now
        killed = pod.stop()
        self.assertEqual(pod, killed)

    def test_999_remove(self):
        global pod

        ident = pod.remove(force=True)
        self.assertEqual(ident, pod.id)

        with self.assertRaises(StopIteration):
            next(self.pclient.pods.list())



import itertools
import os
import unittest
from collections import Counter
from datetime import datetime, timezone
from test.podman_testcase import PodmanTestCase

import podman
from podman import FoldedString


class TestImages(PodmanTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()

    def setUp(self):
        self.tmpdir = os.environ['TMPDIR']
        self.host = os.environ['PODMAN_HOST']

        self.pclient = podman.Client(self.host)
        self.images = self.loadCache()

    def tearDown(self):
        pass

    def loadCache(self):
        with podman.Client(self.host) as pclient:
            self.images = list(pclient.images.list())

        self.alpine_image = next(
            iter([
                i for i in self.images
                if 'docker.io/library/alpine:latest' in i['repoTags']
            ] or []), None)

        return self.images

    def test_list(self):
        actual = self.loadCache()
        self.assertGreaterEqual(len(actual), 2)
        self.assertIsNotNone(self.alpine_image)

    @unittest.skip('TODO: missing buildah json file')
    def test_build(self):
        path = os.path.join(self.tmpdir, 'ctnr', 'Dockerfile')
        img, logs = self.pclient.images.build(
            dockerfile=[path],
            tags=['alpine-unittest'],
        )
        self.assertIsNotNone(img)
        self.assertIn('localhost/alpine-unittest:latest', img.repoTags)
        self.assertLess(
            podman.datetime_parse(img.created), datetime.now(timezone.utc))
        self.assertTrue(logs)

    def test_create(self):
        img_details = self.alpine_image.inspect()

        actual = self.alpine_image.container(command=['sleep',  '1h'])
        self.assertIsNotNone(actual)
        self.assertEqual(FoldedString(actual.status), 'configured')

        ctnr = actual.start()
        self.assertEqual(FoldedString(ctnr.status), 'running')

        ctnr_details = ctnr.inspect()
        for e in img_details.containerconfig['env']:
            self.assertIn(e, ctnr_details.config['env'])

    def test_export(self):
        path = os.path.join(self.tmpdir, 'alpine_export.tar')
        target = 'oci-archive:{}:latest'.format(path)

        actual = self.alpine_image.export(target, False)
        self.assertTrue(actual)
        self.assertTrue(os.path.isfile(path))

    def test_get(self):
        actual = self.pclient.images.get(self.alpine_image.id)
        self.assertEqual(actual, self.alpine_image)

    def test_history(self):
        records = []
        bucket = Counter()
        for record in self.alpine_image.history():
            self.assertIn(record.id, (self.alpine_image.id, '<missing>'))
            bucket[record.id] += 1
            records.append(record)

        self.assertGreater(bucket[self.alpine_image.id], 0)
        self.assertEqual(sum(bucket.values()), len(records))

    def test_inspect(self):
        actual = self.alpine_image.inspect()
        self.assertEqual(actual.id, self.alpine_image.id)

    def test_push(self):
        path = '{}/alpine_push'.format(self.tmpdir)
        target = 'dir:{}'.format(path)
        self.alpine_image.push(target, tlsverify=False)

        self.assertTrue(os.path.isfile(os.path.join(path, 'manifest.json')))
        self.assertTrue(os.path.isfile(os.path.join(path, 'version')))

    def test_tag(self):
        self.assertEqual(self.alpine_image.id,
                         self.alpine_image.tag('alpine:fubar'))
        self.loadCache()
        self.assertIn('localhost/alpine:fubar', self.alpine_image.repoTags)

    def test_remove(self):
        before = self.loadCache()

        # assertRaises doesn't follow the import name :(
        with self.assertRaises(podman.ErrorOccurred):
            self.alpine_image.remove()

        actual = self.alpine_image.remove(force=True)
        self.assertEqual(self.alpine_image.id, actual)
        after = self.loadCache()

        self.assertLess(len(after), len(before))
        TestImages.setUpClass()
        self.loadCache()

    def test_import_delete_unused(self):
        before = self.loadCache()
        # create unused image, so we have something to delete
        source = os.path.join(self.tmpdir, 'alpine_gold.tar')
        new_img = self.pclient.images.import_image(
            source,
            'alpine2:latest',
            'unittest.test_import',
        )
        after = self.loadCache()

        self.assertEqual(len(before) + 1, len(after))
        self.assertIsNotNone(
            next(iter([i for i in after if new_img in i['id']] or []), None))

        actual = self.pclient.images.delete_unused()
        self.assertIn(new_img, actual)

        after = self.loadCache()
        self.assertGreaterEqual(len(before), len(after))

        TestImages.setUpClass()
        self.loadCache()

    def test_pull(self):
        before = self.loadCache()
        actual = self.pclient.images.pull('prom/busybox:latest')
        after = self.loadCache()

        self.assertEqual(len(before) + 1, len(after))
        self.assertIsNotNone(
            next(iter([i for i in after if actual in i['id']] or []), None))

    def test_search(self):
        actual = self.pclient.images.search('alpine', 25)
        names, length = itertools.tee(actual)

        for img in names:
            self.assertIn('alpine', img.name)
        self.assertTrue(0 < len(list(length)) <= 25)


if __name__ == '__main__':
    unittest.main()



"""Remote client command for signaling podman containers."""
import sys

import podman
from pypodman.lib import AbstractActionBase, SignalAction


class Kill(AbstractActionBase):
    """Class for sending signal to main process in container."""

    @classmethod
    def subparser(cls, parent):
        """Add Kill command to parent parser."""
        parser = parent.add_parser('kill', help='signal container')
        parser.add_argument(
            '--signal',
            '-s',
            action=SignalAction,
            default=9,
            help='Signal to send to the container. (default: %(default)s)')
        parser.add_argument(
            'containers',
            nargs='+',
            help='containers to signal',
        )
        parser.set_defaults(class_=cls, method='kill')

    def kill(self):
        """Signal provided containers."""
        try:
            for ident in self._args.containers:
                try:
                    ctnr = self.client.containers.get(ident)
                    ctnr.kill(self._args.signal)
                except podman.ContainerNotFound as e:
                    sys.stdout.flush()
                    print(
                        'Container "{}" not found'.format(e.name),
                        file=sys.stderr,
                        flush=True)
                else:
                    print(ident)
        except podman.ErrorOccurred as e:
            sys.stdout.flush()
            print(
                '{}'.format(e.reason).capitalize(),
                file=sys.stderr,
                flush=True)
            return 1



"""Base class for all actions of remote client."""
import abc
from functools import lru_cache

import podman


class AbstractActionBase(abc.ABC):
    """Base class for all actions of remote client."""

    @classmethod
    @abc.abstractmethod
    def subparser(cls, parent):
        """Define parser for this action.  Subclasses must implement.

        API:
        Use set_defaults() to set attributes "class_" and "method". These will
        be invoked as class_(parsed_args).method()
        """
        parent.add_flag(
            '--all',
            help='list all items.')
        parent.add_flag(
            '--truncate',
            '--trunc',
            default=True,
            help="Truncate id's and other long fields.")
        parent.add_flag(
            '--heading',
            default=True,
            help='Include table headings in the output.')
        parent.add_flag(
            '--quiet',
            help='List only the IDs.')

    def __init__(self, args):
        """Construct class."""
        # Dump all unset arguments before transmitting to service
        self._args = args
        self.opts = {
            k: v
            for k, v in vars(self._args).items() if v is not None
        }

    @property
    def remote_uri(self):
        """URI for remote side of connection."""
        return self._args.remote_uri

    @property
    def local_uri(self):
        """URI for local side of connection."""
        return self._args.local_uri

    @property
    def identity_file(self):
        """Key for authenication."""
        return self._args.identity_file

    @property
    @lru_cache(maxsize=1)
    def client(self):
        """Podman remote client for communicating."""
        if self._args.host is None:
            return podman.Client(uri=self.local_uri)
        return podman.Client(
            uri=self.local_uri,
            remote_uri=self.remote_uri,
            identity_file=self.identity_file)

    def __repr__(self):
        """Compute the “official” string representation of object."""
        return ("{}(local_uri='{}', remote_uri='{}',"
                " identity_file='{}')").format(
                    self.__class__,
                    self.local_uri,
                    self.remote_uri,
                    self.identity_file,
                )



#!/usr/bin/env python

import os

from setuptools import find_packages, setup


root = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(root, 'README.md')) as me:
    readme = me.read()

with open(os.path.join(root, 'requirements.txt')) as r:
    requirements = r.read().splitlines()


setup(
    name='podman',
    version=os.environ.get('PODMAN_VERSION', '0.0.0'),
    description='A library for communicating with a Podman server',
    author='Jhon Honce',
    author_email='jhonce@redhat.com',
    license='Apache Software License',
    long_description=readme,
    include_package_data=True,
    install_requires=requirements,
    packages=find_packages(exclude=['test']),
    python_requires='>=3',
    zip_safe=True,
    url='http://github.com/containers/libpod',
    keywords='varlink libpod podman',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development',
    ])



"""Remote client command for pausing processes in containers."""
import sys

import podman
from pypodman.lib import AbstractActionBase


class Pause(AbstractActionBase):
    """Class for pausing processes in container."""

    @classmethod
    def subparser(cls, parent):
        """Add Pause command to parent parser."""
        parser = parent.add_parser('pause', help='pause container processes')
        parser.add_argument(
            'containers',
            nargs='+',
            help='containers to pause',
        )
        parser.set_defaults(class_=cls, method='pause')

    def pause(self):
        """Pause provided containers."""
        try:
            for ident in self._args.containers:
                try:
                    ctnr = self.client.containers.get(ident)
                    ctnr.pause()
                except podman.ContainerNotFound as e:
                    sys.stdout.flush()
                    print(
                        'Container "{}" not found'.format(e.name),
                        file=sys.stderr,
                        flush=True)
                else:
                    print(ident)
        except podman.ErrorOccurred as e:
            sys.stdout.flush()
            print(
                '{}'.format(e.reason).capitalize(),
                file=sys.stderr,
                flush=True)
            return 1



#!/usr/bin/env python3
"""Example: Show all images on system."""

import podman

print('{}\n'.format(__doc__))

with podman.Client() as client:
    for img in client.images.list():
        print(img)



"""Remote client command for creating pod."""
import sys

import podman
from pypodman.lib import AbstractActionBase


class CreatePod(AbstractActionBase):
    """Implement Create Pod command."""

    @classmethod
    def subparser(cls, parent):
        """Add Pod Create command to parent parser."""
        parser = parent.add_parser('create', help='create pod')
        super().subparser(parser)

        parser.add_argument(
            '--cgroup-parent',
            dest='cgroupparent',
            type=str,
            help='Path to cgroups under which the'
            ' cgroup for the pod will be created.')
        parser.add_flag(
            '--infra',
            help='Create an infra container and associate it with the pod.')
        parser.add_argument(
            '-l',
            '--label',
            dest='labels',
            action='append',
            type=str,
            help='Add metadata to a pod (e.g., --label=com.example.key=value)')
        parser.add_argument(
            '-n',
            '--name',
            dest='ident',
            type=str,
            help='Assign name to the pod')
        parser.add_argument(
            '--share',
            choices=('ipc', 'net', 'pid', 'user', 'uts'),
            help='Comma deliminated list of kernel namespaces to share')

        parser.set_defaults(class_=cls, method='create')

        # TODO: Add golang CLI arguments not included in API.
        # parser.add_argument(
        #     '--infra-command',
        #     default='/pause',
        #     help='Command to run to start the infra container.'
        #     '(default: %(default)s)')
        # parser.add_argument(
        #     '--infra-image',
        #     default='k8s.gcr.io/pause:3.1',
        #     help='Image to create for the infra container.'
        #     '(default: %(default)s)')
        # parser.add_argument(
        #     '--podidfile',
        #     help='Write the pod ID to given file name on remote host')

    def create(self):
        """Create Pod from given options."""
        config = {}
        for key in ('ident', 'cgroupparent', 'infra', 'labels', 'share'):
            config[key] = self.opts.get(key)

        try:
            pod = self.client.pods.create(**config)
        except podman.ErrorOccurred as ex:
            sys.stdout.flush()
            print(
                '{}'.format(ex.reason).capitalize(),
                file=sys.stderr,
                flush=True)
        else:
            print(pod.id)



from __future__ import absolute_import

import unittest
from unittest.mock import patch

import podman
from podman.client import BaseClient, Client, LocalClient, RemoteClient


class TestClient(unittest.TestCase):
    def setUp(self):
        pass

    @patch('podman.libs.system.System.ping', return_value=True)
    def test_local(self, mock_ping):
        p = Client(
            uri='unix:/run/podman',
            interface='io.podman',
        )

        self.assertIsInstance(p._client, LocalClient)
        self.assertIsInstance(p._client, BaseClient)

        mock_ping.assert_called_once_with()

    @patch('podman.libs.system.System.ping', return_value=True)
    def test_remote(self, mock_ping):
        p = Client(
            uri='unix:/run/podman',
            interface='io.podman',
            remote_uri='ssh://user@hostname/run/podman/podman',
            identity_file='~/.ssh/id_rsa')

        self.assertIsInstance(p._client, BaseClient)
        mock_ping.assert_called_once_with()



"""Remote client command for pulling images."""
import sys

import podman
from pypodman.lib import AbstractActionBase


class Pull(AbstractActionBase):
    """Class for retrieving images from repository."""

    @classmethod
    def subparser(cls, parent):
        """Add Pull command to parent parser."""
        parser = parent.add_parser(
            'pull',
            help='retrieve image from repository',
        )
        parser.add_argument(
            'targets',
            nargs='+',
            help='image id(s) to retrieve.',
        )
        parser.set_defaults(class_=cls, method='pull')

    def __init__(self, args):
        """Construct Pull class."""
        super().__init__(args)

    def pull(self):
        """Retrieve image."""
        for ident in self._args.targets:
            try:
                self.client.images.pull(ident)
                print(ident)
            except podman.ImageNotFound as e:
                sys.stdout.flush()
                print(
                    'Image {} not found.'.format(e.name),
                    file=sys.stderr,
                    flush=True)
            except podman.ErrorOccurred as e:
                sys.stdout.flush()
                print(
                    '{}'.format(e.reason).capitalize(),
                    file=sys.stderr,
                    flush=True)



"""Remote client command for retrieving ports from containers."""
import sys

import podman
from pypodman.lib import AbstractActionBase


class Port(AbstractActionBase):
    """Class for retrieving ports from container."""

    @classmethod
    def subparser(cls, parent):
        """Add Port command to parent parser."""
        parser = parent.add_parser(
            'port', help='retrieve ports from containers')
        parser.add_flag(
            '--all',
            '-a',
            help='List all known port mappings for running containers')
        parser.add_argument(
            'containers',
            nargs='*',
            help='containers to list ports',
        )
        parser.set_defaults(class_=cls, method='port')

    def __init__(self, args):
        """Construct Port class."""
        if not args.all and not args.containers:
            raise ValueError('You must supply at least one'
                             ' container id or name, or --all.')
        super().__init__(args)

    def port(self):
        """Retrieve ports from containers."""
        try:
            ctnrs = []
            if self._args.all:
                ctnrs = self.client.containers.list()
            else:
                for ident in self._args.containers:
                    try:
                        ctnrs.append(self.client.containers.get(ident))
                    except podman.ContainerNotFound as e:
                        sys.stdout.flush()
                        print(
                            'Container "{}" not found'.format(e.name),
                            file=sys.stderr,
                            flush=True)

            for ctnr in ctnrs:
                print("{}\n{}".format(ctnr.id, ctnr.ports))

        except podman.ErrorOccurred as e:
            sys.stdout.flush()
            print(
                '{}'.format(e.reason).capitalize(),
                file=sys.stderr,
                flush=True)
            return 1
        return 0



"""Remote client for reporting image history."""
import json
from collections import OrderedDict

import humanize

import podman
from pypodman.lib import AbstractActionBase, Report, ReportColumn


class History(AbstractActionBase):
    """Class for reporting Image History."""

    @classmethod
    def subparser(cls, parent):
        """Add History command to parent parser."""
        parser = parent.add_parser('history', help='report image history')
        super().subparser(parser)
        parser.add_flag(
            '--human',
            '-H',
            help='Display sizes and dates in human readable format.')
        parser.add_argument(
            '--format',
            choices=('json', 'table'),
            help="Alter the output for a format like 'json' or 'table'."
            " (default: table)")
        parser.add_argument(
            'image', nargs='+', help='image for history report')
        parser.set_defaults(class_=cls, method='history')

    def __init__(self, args):
        """Construct History class."""
        super().__init__(args)

        self.columns = OrderedDict({
            'id':
            ReportColumn('id', 'ID', 12),
            'created':
            ReportColumn('created', 'CREATED', 11),
            'createdBy':
            ReportColumn('createdBy', 'CREATED BY', 45),
            'size':
            ReportColumn('size', 'SIZE', 8),
            'comment':
            ReportColumn('comment', 'COMMENT', 0)
        })

    def history(self):
        """Report image history."""
        rows = list()
        for ident in self._args.image:
            for details in self.client.images.get(ident).history():
                fields = dict(details._asdict())

                if self._args.human:
                    fields.update({
                        'size':
                        humanize.naturalsize(details.size),
                        'created':
                        humanize.naturaldate(
                            podman.datetime_parse(details.created)),
                    })
                del fields['tags']

                rows.append(fields)

        if self._args.quiet:
            for row in rows:
                ident = row['id'][:12] if self._args.truncate else row['id']
                print(ident)
        elif self._args.format == 'json':
            print(json.dumps(rows, indent=2), flush=True)
        else:
            with Report(self.columns, heading=self._args.heading) as report:
                report.layout(
                    rows, self.columns.keys(), truncate=self._args.truncate)
                for row in rows:
                    report.row(**row)



#!/usr/bin/env python3
"""Example: Show containers grouped by image id."""

from itertools import groupby

import podman

print('{}\n'.format(__doc__))

with podman.Client() as client:
    ctnrs = sorted(client.containers.list(), key=lambda k: k.imageid)
    for key, grp in groupby(ctnrs, key=lambda k: k.imageid):
        print('Image: {}'.format(key))
        for c in grp:
            print('     : container: {} created at: {}'.format(
                c.id[:12], podman.datetime_format(c.createdat)))



"""Remote client command for reporting on Podman service."""
import sys

import podman
from pypodman.lib import AbstractActionBase


class Version(AbstractActionBase):
    """Class for reporting on Podman Service."""

    @classmethod
    def subparser(cls, parent):
        """Add Version command to parent parser."""
        parser = parent.add_parser(
            'version', help='report version on podman service')
        parser.set_defaults(class_=cls, method='version')

    def version(self):
        """Report on Podman Service."""
        try:
            info = self.client.system.info()
        except podman.ErrorOccurred as e:
            sys.stdout.flush()
            print(
                '{}'.format(e.reason).capitalize(),
                file=sys.stderr,
                flush=True)
            return 1
        else:
            version = info._asdict()['podman']
            host = info._asdict()['host']
            print("Version        {}".format(version['podman_version']))
            print("Go Version     {}".format(version['go_version']))
            print("Git Commit     {}".format(version['git_commit']))
            print("OS/Arch        {}/{}".format(host["os"], host["arch"]))



"""Remote client command to import tarball as image filesystem."""
import sys

import podman
from pypodman.lib import AbstractActionBase, ChangeAction


class Import(AbstractActionBase):
    """Class for importing tarball as image filesystem."""

    @classmethod
    def subparser(cls, parent):
        """Add Import command to parent parser."""
        parser = parent.add_parser(
            'import',
            help='import tarball as image filesystem',
        )
        parser.add_argument(
            '--change',
            '-c',
            action=ChangeAction,
        )
        parser.add_argument(
            '--message',
            '-m',
            help='Set commit message for imported image.',
        )
        parser.add_argument(
            'source',
            metavar='PATH',
            nargs=1,
            help='tarball to use as source on remote system',
        )
        parser.add_argument(
            'reference',
            metavar='TAG',
            nargs='*',
            help='Optional tag for image. (default: None)',
        )
        parser.set_defaults(class_=cls, method='import_')

    def import_(self):
        """Import tarball as image filesystem."""
        # ImportImage() validates it's parameters therefore we need to create
        # pristine dict() for keywords
        options = {}
        if 'message' in self.opts:
            options['message'] = self.opts['message']
        if 'change' in self.opts and self.opts['change']:
            options['changes'] = self.opts['change']

        reference = self.opts['reference'][0] if 'reference' in self.opts\
            else None

        try:
            ident = self.client.images.import_image(
                self.opts['source'][0],
                reference,
                **options,
            )
            print(ident)
        except podman.ErrorOccurred as e:
            sys.stdout.flush()
            print(
                '{}'.format(e.reason).capitalize(),
                file=sys.stderr,
                flush=True)
            return 1
        return 0



"""Remote client command for signaling pods and their containers."""
import signal
import sys

import podman
from pypodman.lib import AbstractActionBase, SignalAction
from pypodman.lib import query_model as query_pods


class KillPod(AbstractActionBase):
    """Class for sending signal to processes in pod."""

    @classmethod
    def subparser(cls, parent):
        """Add Pod Kill command to parent parser."""
        parser = parent.add_parser('kill', help='signal containers in pod')

        parser.add_flag(
            '--all',
            '-a',
            help='Sends signal to all pods.')
        parser.add_argument(
            '-s',
            '--signal',
            action=SignalAction,
            default=9,
            help='Signal to send to the pod. (default: %(default)s)')
        parser.add_argument('pod', nargs='*', help='pod(s) to signal')
        parser.set_defaults(class_=cls, method='kill')

    def __init__(self, args):
        """Construct Pod Kill object."""
        if args.all and args.pod:
            raise ValueError('You may give a pod or use --all, but not both')
        super().__init__(args)

    def kill(self):
        """Signal provided pods."""
        idents = None if self._args.all else self._args.pod
        pods = query_pods(self.client.pods, idents)

        for pod in pods:
            try:
                pod.kill(self._args.signal)
                print(pod.id)
            except podman.PodNotFound as ex:
                print(
                    'Pod "{}" not found.'.format(ex.name),
                    file=sys.stderr,
                    flush=True)
            except podman.ErrorOccurred as e:
                print(
                    '{}'.format(e.reason).capitalize(),
                    file=sys.stderr,
                    flush=True)
                return 1
        return 0



"""Remote client command for deleting images."""
import sys

import podman
from pypodman.lib import AbstractActionBase


class Rmi(AbstractActionBase):
    """Class for removing images from storage."""

    @classmethod
    def subparser(cls, parent):
        """Add Rmi command to parent parser."""
        parser = parent.add_parser('rmi', help='delete image(s)')
        parser.add_flag(
            '--force',
            '-f',
            help='force delete of image(s) and associated containers.')
        parser.add_argument('targets', nargs='+', help='image id(s) to delete')
        parser.set_defaults(class_=cls, method='remove')

    def remove(self):
        """Remove image(s)."""
        for ident in self._args.targets:
            try:
                img = self.client.images.get(ident)
                img.remove(self._args.force)
                print(ident)
            except podman.ImageNotFound as e:
                sys.stdout.flush()
                print(
                    'Image {} not found.'.format(e.name),
                    file=sys.stderr,
                    flush=True)
            except podman.ErrorOccurred as e:
                sys.stdout.flush()
                print(
                    '{}'.format(e.reason).capitalize(),
                    file=sys.stderr,
                    flush=True)



"""Remote podman client."""
import logging
import os
import sys
from subprocess import CalledProcessError

from pypodman.lib import PodmanArgumentParser


def main():
    """Entry point."""
    # Setup logging so we use stderr and can change logging level later
    # Do it now before there is any chance of a default setup hardcoding crap.
    log = logging.getLogger()
    fmt = logging.Formatter('%(asctime)s | %(levelname)-8s | %(message)s',
                            '%Y-%m-%d %H:%M:%S %Z')
    stderr = logging.StreamHandler(stream=sys.stderr)
    stderr.setFormatter(fmt)
    log.addHandler(stderr)
    log.setLevel(logging.WARNING)

    parser = PodmanArgumentParser()
    args = parser.parse_args()

    log.setLevel(args.log_level)
    logging.debug(
        'Logging initialized at level %s',
        logging.getLevelName(logging.getLogger().getEffectiveLevel()))

    def want_tb():
        """Add traceback when logging events."""
        return log.getEffectiveLevel() == logging.DEBUG

    try:
        if not os.path.exists(args.run_dir):
            os.makedirs(args.run_dir)
    except PermissionError as e:
        logging.critical(e, exc_info=want_tb())
        sys.exit(6)

    # class_(args).method() are set by the sub-command's parser
    returncode = None
    try:
        obj = args.class_(args)
    except AttributeError:
        parser.print_help(sys.stderr)
        sys.exit(1)
    except ValueError as e:
        print(e, file=sys.stderr, flush=True)
        sys.exit(1)
    except Exception as e:  # pylint: disable=broad-except
        logging.critical(repr(e), exc_info=want_tb())
        logging.warning('See subparser "%s" configuration.',
                        args.subparser_name)
        sys.exit(5)

    try:
        returncode = getattr(obj, args.method)()
    except KeyboardInterrupt:
        pass
    except AttributeError as e:
        logging.critical(e, exc_info=want_tb())
        logging.warning('See subparser "%s" configuration.',
                        args.subparser_name)
        returncode = 3
    except (
            CalledProcessError,
            ConnectionError,
            ConnectionRefusedError,
            ConnectionResetError,
            TimeoutError,
    ) as e:
        logging.critical(e, exc_info=want_tb())
        logging.info('Review connection arguments for correctness.')
        returncode = 4

    return 0 if returncode is None else returncode


if __name__ == '__main__':
    sys.exit(main())



import datetime
import unittest

import podman


class TestLibs(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_parse(self):
        expected = datetime.datetime.strptime(
            '2018-05-08T14:12:53.797795-0700', '%Y-%m-%dT%H:%M:%S.%f%z')
        for v in [
                '2018-05-08T14:12:53.797795191-07:00',
                '2018-05-08T14:12:53.797795-07:00',
                '2018-05-08T14:12:53.797795-0700',
                '2018-05-08 14:12:53.797795191 -0700 MST',
        ]:
            actual = podman.datetime_parse(v)
            self.assertEqual(actual, expected)

        expected = datetime.datetime.strptime(
            '2018-05-08T14:12:53.797795-0000', '%Y-%m-%dT%H:%M:%S.%f%z')
        for v in [
                '2018-05-08T14:12:53.797795191Z',
                '2018-05-08T14:12:53.797795191z',
        ]:
            actual = podman.datetime_parse(v)
            self.assertEqual(actual, expected)

        actual = podman.datetime_parse(datetime.datetime.now().isoformat())
        self.assertIsNotNone(actual)

    def test_parse_fail(self):
        for v in [
                'There is no time here.',
        ]:
            with self.assertRaises(ValueError):
                podman.datetime_parse(v)

    def test_format(self):
        expected = '2018-05-08T18:24:52.753227-07:00'
        dt = podman.datetime_parse(expected)
        actual = podman.datetime_format(dt)
        self.assertEqual(actual, expected)


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import

import logging
import time
import unittest
from unittest.mock import MagicMock, patch

from podman.libs.tunnel import Context, Portal, Tunnel


class TestTunnel(unittest.TestCase):
    def setUp(self):
        self.tunnel_01 = MagicMock(spec=Tunnel)
        self.tunnel_02 = MagicMock(spec=Tunnel)

    def test_portal_ops(self):
        portal = Portal(sweap=500)
        portal['unix:/01'] = self.tunnel_01
        portal['unix:/02'] = self.tunnel_02

        self.assertEqual(portal.get('unix:/01'), self.tunnel_01)
        self.assertEqual(portal.get('unix:/02'), self.tunnel_02)

        del portal['unix:/02']
        with self.assertRaises(KeyError):
            portal['unix:/02']
        self.assertEqual(len(portal), 1)

    def test_portal_reaping(self):
        portal = Portal(sweap=0.5)
        portal['unix:/01'] = self.tunnel_01
        portal['unix:/02'] = self.tunnel_02

        self.assertEqual(len(portal), 2)
        for entry in portal:
            self.assertIn(entry, (self.tunnel_01, self.tunnel_02))

        time.sleep(1)
        portal.reap()
        self.assertEqual(len(portal), 0)

    def test_portal_no_reaping(self):
        portal = Portal(sweap=500)
        portal['unix:/01'] = self.tunnel_01
        portal['unix:/02'] = self.tunnel_02

        portal.reap()
        self.assertEqual(len(portal), 2)
        for entry in portal:
            self.assertIn(entry, (self.tunnel_01, self.tunnel_02))

    @patch('subprocess.Popen')
    @patch('os.path.exists', return_value=True)
    @patch('weakref.finalize')
    def test_tunnel(self, mock_finalize, mock_exists, mock_Popen):
        mock_Popen.return_value.returncode = 0

        context = Context(
            'unix:/01',
            'io.podman',
            '/tmp/user/socket',
            '/run/podman/socket',
            'user',
            'hostname',
            None,
            '~/.ssh/id_rsa',
        )
        tunnel = Tunnel(context).bore()

        cmd = ['ssh', '-fNT']
        if logging.getLogger().getEffectiveLevel() == logging.DEBUG:
            cmd.append('-v')
        else:
            cmd.append('-q')

        cmd.extend((
            '-L',
            '{}:{}'.format(context.local_socket, context.remote_socket),
            '-i',
            context.identity_file,
            '{}@{}'.format(context.username, context.hostname),
        ))

        mock_finalize.assert_called_once_with(tunnel, tunnel.close)
        mock_exists.assert_called_once_with(context.local_socket)
        mock_Popen.assert_called_once_with(cmd, close_fds=True)



#!/usr/bin/env python3
"""Example: Show all containers created since midnight."""

from datetime import datetime, time, timezone

import podman

print('{}\n'.format(__doc__))


midnight = datetime.combine(datetime.today(), time.min, tzinfo=timezone.utc)

with podman.Client() as client:
    for c in client.containers.list():
        created_at = podman.datetime_parse(c.createdat)

        if created_at > midnight:
            print('{}: image: {} createdAt: {}'.format(
                c.id[:12], c.image[:32], podman.datetime_format(created_at)))



"""Remote podman client support library."""
import sys

import podman
from pypodman.lib.action_base import AbstractActionBase
from pypodman.lib.parser_actions import (ChangeAction, PathAction,
                                         PositiveIntAction, SignalAction,
                                         UnitAction)
from pypodman.lib.podman_parser import PodmanArgumentParser
from pypodman.lib.report import Report, ReportColumn

# Silence pylint overlording...
assert ChangeAction
assert PathAction
assert PositiveIntAction
assert SignalAction
assert UnitAction

__all__ = [
    'AbstractActionBase',
    'PodmanArgumentParser',
    'Report',
    'ReportColumn',
]


def query_model(model, identifiers=None):
    """Retrieve all (default) or given model(s)."""
    objs = []
    if identifiers is None:
        objs.extend(model.list())
    else:
        try:
            for ident in identifiers:
                objs.append(model.get(ident))
        except (
                podman.PodNotFound,
                podman.ImageNotFound,
                podman.ContainerNotFound,
        ) as ex:
            print(
                '"{}" not found'.format(ex.name), file=sys.stderr, flush=True)
    return objs
